#delimit;

*A model of tenure decisions made based on average performance over n years;

*Calls:
* sub_makeinitialgrid;

cap program drop learningmodel_tenure;
program define learningmodel_tenure, rclass;
*Program for a model where teachers are fired after n years if average 
*performance to date falls below a threshold;
syntax using/,  
         /*Teacher career parameters*/
           basehazard(string) maxt(real) expreturn(real) fixedcost_base(real) fixedcost_policy(real) 
         /*Teacher effectiveness parameters*/
           sigma_mu(real) expeffect(string) h(real) corrtruth(real)
         /*Performance measures*/
           rsignal(real)
         /*Tenure policy*/
           frtenured(real) tenuredate(real) fireeffect(string) policybase(real)
         /*Preference parameters*/
           crra(real) crra_type(string) discount(real) elast_exit(string) elast_entry(string) maxwage(real)
         /*Simulation parameters*/
           gridpts(integer) simulobs(integer)
         /*Base contract parameters*/
           firethreshold(real) sdeffect_base(real) vareffect_base(real) avgeffect_base(real) meanwage_base(real) 
           nteach_base(real) elast_valwage(real) initialvalue(real) eta_exit(name) val_fired(name)
         [*];
  *This program has one parameter more than in the outer program:
  *  policybase:  The base pay under the firing policy as a fraction of the base under the base contract;
  *Set defaults for non-numeric parameters;         
 clear;
 clear results;

 *Generate a new local variable, since this has been renamed;
 local firebase=`policybase';

 *Some calculations;
  local var_mu=`sigma_mu'^2;
  local var_e=`var_mu'*(1-`rsignal')/`rsignal';
  local sigma_e=sqrt(`var_e');
  local var_sig=`var_mu'+`var_e';
  local sigma_sig=sqrt(`var_sig');
  local priorvar0=`var_mu'*(1-`h');
  local var_prior0=`var_mu'*`h';
  *Extract the c and eta parameters;
   forvalues t=2/`maxt' {;
  	 local eta`t'=el(`eta_exit', `t', 2);
  	 local c`t'=el(`eta_exit', `t', 3);
   };

 *Work out the variance of the prior at each year -- will use this to make grid points;
   *At period t, have t-1 signals, on top of initial prior tau with mu|tau ~ N(tau, (1-h)s2_mu);
   *So posterior variance is [((1-h)s2_mu)^-1 + (s2_e/T)^-1]^-1;
 local priorvar_1=(1-`h')*`var_mu';
 di "priorvar_1 is `priorvar_1'";
 local dpriorvar_1=`var_mu';
 forvalues t=2/`maxt' {;
 	 local s=`t'-1;
 	 local priorvar_`t'=1/( (1/`priorvar_`s'') + (1/`var_e'));
 	 local dpriorvar_`t'=1/( (1/`dpriorvar_`s'') + (1/`var_e'));
 };
 forvalues t=1/`maxt' {;
   local varprior_`t'=`var_mu'-`priorvar_`t'';
 	 local priorsd_`t'=sqrt(`priorvar_`t'');
 	 local sdprior_`t'=sqrt(`varprior_`t'');
   local dvarprior_`t'=`var_mu'-`dpriorvar_`t'';
 	 local dpriorsd_`t'=sqrt(`dpriorvar_`t'');
 	 local dsdprior_`t'=sqrt(`dvarprior_`t'');
 };

**Make a grid of initial pvt info - signal values, evenly spaced in the distribution;
 tempfile priorpts;
 sub_makeinitialgrid, gridpts(`gridpts') var_prior0(`var_prior0') maxt(`maxt') priorvar_1(`priorvar_1') 
                      var_e(`var_e') var_mu(`var_mu') ctype("tenure");
 save `priorpts';
 
**Now need to turn to the actual learning model / value function evaluation.;
***Start with a very simple model where no one is fired (so signals are irrelevant),
***to compute baseline value;
   drop _all;
   set obs 1;
   gen wage_policy=`firebase'*((1+`expreturn')^(`maxt'-1));
   gen exper=`maxt'-1;
   gen basehaz=`basehazard';
   su basehaz, meanonly;
   local baseh=r(mean);
   sub_valfn, crra(`crra') etanext(0) next(`maxt') basehazard(`baseh') cnext(0)
              maxwage(`maxwage') discount(`discount') crra_type(`crra_type') lastperiod;
   local baseval`maxt'=valfn_`maxt';
   forvalues n=2/`maxt' {;
   	 local t=`maxt'-`n'+1;
   	 local next=`t'+1;
   	 replace wage_policy=`firebase'*((1+`expreturn')^(`t'-1));
     replace exper=`t'-1;
     replace basehaz=`basehazard';
     su basehaz, meanonly;
     local baseh=r(mean);
     sub_valfn, crra(`crra') etanext(`eta`next'') next(`next') basehazard(`baseh') cnext(`c`next'')
                maxwage(`maxwage') discount(`discount') crra_type(`crra_type');
     local baseval`t'=valfn_`t';           
   };
 *And extract continuation values for those who are fired;
  drop _all;
  set obs `maxt';
  svmat `val_fired', names(col);
  sort exper;
  assert exper[1]==0;
  assert exper==exper[_n-1]+1 if _n>1;
  forvalues t=1/`maxt' {;
  	local valfired_`t'=valfn_fired[`t'];
  };
    
***Now start over with the real value evaluation;
***Note that in tenure policy, value function is invariant to ability after the tenure decision is made.;
***This makes things simple!;
***Start with period T;
   *Value function entering period T, assuming not yet fired;
    drop _all;
    set obs 1;
    local s=`maxt'-1;
    gen wage_policy=`firebase'*(1+`expreturn')^`s';
    gen exper=`maxt'-1;
    gen basehaz=`basehazard';
    su basehaz, meanonly;
    local baseh=r(mean);
    sub_valfn, crra(`crra') etanext(0) next(`maxt') basehazard(`baseh') cnext(0)
              maxwage(`maxwage') discount(`discount') crra_type(`crra_type') lastperiod;
    gen hazard_`s'=(`basehazard')*(`c`maxt''/valfn_`maxt')^`eta`maxt'';
    *gen hazard_`s'=`basehazard' ; *Hazard of voluntary exit between T-1 and T;
    local valfn_`maxt'=valfn_`maxt';
    local hazard_`s'=hazard_`s';
***Now period t;
   local penultimate=`maxt'-1;
   local first=1;
*local first=28;   
   forvalues t=`penultimate' (-1) `first' {; 
   	 local next=`t'+1;  
   	 local s=`t'-1; 
   	 di "STARTING t=`t'";
 	   *Can save a lot of time post tenure decision, where signals dont matter;
 	   if `t'>`tenuredate' {;
 	   	 drop _all;
 	   	 set obs 1;
       gen wage_policy=`firebase'*(1+`expreturn')^`s';
       ge exper=`t'-1;
       gen basehaz=`basehazard';
       su basehaz, meanonly;
       local baseh=r(mean);
       gen valfn_`next'=`valfn_`next'';
       sub_valfn, crra(`crra') etanext(`eta`next'') next(`next') basehazard(`baseh') cnext(`c`next'')
                 maxwage(`maxwage') discount(`discount') crra_type(`crra_type') 
                 ;
       if `s'>0 gen hazard_`s'=(`basehazard')*(`c`t''/valfn_`t')^`eta`t'';
       else gen hazard_`s'=0;
       local valfn_`t'=valfn_`t';
       local hazard_`s'=hazard_`s';
 	   };
 	   *But if not yet tenured, need to keep track of things;
 	   else {; 
       use g_pvtinfo g_avgsig pvtinfo prior`t' avgva`t' using `priorpts', clear;
 	     gen wage_policy=`firebase'*((1+`expreturn')^(`t'-1));
       *Generate distribution of VA draws for period t;
        expand `gridpts';
        sort g_pvtinfo g_avgsig;
        by g_pvtinfo g_avgsig: gen g_nextsig=_n;
        by g_pvtinfo g_avgsig: 
          gen draw=prior`t' + sqrt(`priorvar_`t'' + `var_e')*invnorm(g_nextsig/(`gridpts'+1));
       *Update priors;
        gen avgva=(`s'*avgva`t' + draw)/`t';
        gen avgsig=avgva-pvtinfo;
   	    *gen dposterior=avgsig*`var_mu'/(`var_mu' + (`var_e'/`t'));
 	      *gen posterior=pvtinfo + (avgsig-pvtinfo)*`priorvar_1'/(`priorvar_1' + (`var_e'/`t'));
       *Make firing decision;
        if `t'==`tenuredate' gen byte fired_`t'=(avgva<=`firethreshold');
        else gen byte fired_`t'=0;
       *Rename variables;
        rename g_avgsig prior_g_avgsig;
 	     *Round posterior to enable matching;
 	      gen g_avgsig`next'=round((`gridpts'+1)*normal(avgsig/sqrt(`priorvar_1' + (`var_e'/`t'))));
 	      replace g_avgsig`next'=1 if g_avgsig`next'==0;
 	      replace g_avgsig`next'=`gridpts' if g_avgsig`next'==`gridpts'+1;
       *Merge;
        if `t'<`tenuredate' {;
          merge m:1 g_pvtinfo g_avgsig`next' using `value_`next'', assert(2 3) keep(3) 
                keepusing(g_pvtinfo g_avgsig`next' valfn_`next' hazard_`t');
          drop _merge;
        };
        else {;
        	gen valfn_`next'=`valfn_`next'';
        	gen hazard_`t'=`hazard_`t'';
        }; 
       *Replace next periods value with penalty value if fired this period;
        replace valfn_`next'=`valfired_`next'' if fired_`t'==1;
       *Calculate certainty-equivalent of next periods utility, taking account of possibility of exit
       *to outside option next period.;
        gen exper=`t'-1;
        gen basehaz=`basehazard';
        su basehaz, meanonly;
        local baseh=r(mean);
        sub_valfn, crra(`crra') etanext(`eta`next'') next(`next') basehazard(`baseh') cnext(`c`next'')
                 maxwage(`maxwage') discount(`discount') crra_type(`crra_type') extracollapse(fired_`t') 
                 collapseby(g_pvtinfo prior_g_avgsig );
       rename fired_`t' prfired_`t';
       rename prior_g_avgsig g_avgsig`t';
       if `s'>0 {;
       	 gen exper=`s';
       	 gen hazard_`s'=(`basehazard')*(`c`t''/valfn_`t')^`eta`t'';
       	 drop exper;
       };
       else gen hazard_`s'=0;
       cap drop wage_policy;
       cap drop ra_thisperiod;
       cap drop exper;
       sort g_pvtinfo g_avgsig`t';
       compress;
       tempfile value_`t';
       save `value_`t'';
     };
    };

*Generate some sample paths and merge to value function to get hazards;
 use id pvtinfo mu muB va* avgva* g_pvtinfo g_avgsig* using `using';
 gen byte firedyet_0=0;
 gen byte fired_0=0;
 forvalues t=1/`maxt' {;
 	local s=`t'-1;
 	gen byte firedyet_`t'=firedyet_`s'+fired_`s'; *Fired before start of period t;
 	gen byte fired_`t'=(avgva`t'<=`firethreshold')*(firedyet_`t'==0)*(`t'==`tenuredate'); *Fired after period t;
 };
 drop firedyet_0 fired_0 ;
 	
*Merge these to the value function data to get hazards;
 gen exper=.;
 forvalues t=1/`maxt' {;
 	local s=`t'-1;
 	replace exper=`s';
 	if `t'<=`tenuredate' {;
 		 merge m:1 g_pvtinfo g_avgsig`t' using `value_`t'', assert(2 3) keep(3);
  };
 	else {;
 		gen valfn_`t'=`valfn_`t'';
 		gen hazard_`s'=`hazard_`s'';
 		gen prfired_`t'=0;
 	};
 	*replace valfn_`t'=`baseval`t''*(1+`fireeffect') if firedyet_`t'==1;
 	if `t'==1 {;
 		gen Surv1=1;
 		gen Surv_policy1=1;
 		gen entryweight=((valfn_1-`fixedcost_policy')/(`initialvalue'-`fixedcost_base'))^((`elast_entry')/`elast_valwage');
 	};
 	else {;
 		local s=`t'-1;
 		gen Surv`t'=Surv`s'*(1-(`basehazard'));
 		gen Surv_policy`t'=Surv_policy`s'*(1-hazard_`s')*(firedyet_`t'==0);
 		*drop hazard_`s';
  };
 	cap drop _merge; 
 };
 drop exper;

*Compute the information we need without reshaping -- # of teachers & total wages;
 local nteach_base=0;
 local totsal_base=0;
 local nteach_policy=0;
 local totsal_policy=0;
 gen weight_policy=.;
 forvalues t=1/`maxt' {;
 	 su Surv`t', meanonly;
 	 local nteach_base=`nteach_base'+r(mean)/`maxt';
 	 local totsal_base=`totsal_base' + r(mean)*((1+`expreturn')^(`t'-1))/`maxt';
 	 replace weight_policy=entryweight*Surv_policy`t';
 	 su weight_policy, meanonly;
 	 local nteach_policy=`nteach_policy'+r(mean)/`maxt';
 	 local totsal_policy=`totsal_policy' + r(mean)*`firebase'*((1+`expreturn')^(`t'-1))/`maxt';
 };
 drop weight_policy;

 return scalar nteach_base=`nteach_base';
 return scalar nteach_policy=`nteach_policy';
 return scalar totsal_base=`totsal_base';
 return scalar totsal_policy=`totsal_policy';
 	 
/*
*Reshape to make a teacher-by-year data set;
 reshape long avgva prfired_ fired_ firedyet_ Surv Surv_policy valfn_ hazard_, i(id) j(year);
 drop if year>`maxt';
 drop if year<1;
 gen byte exper=year-1;
 gen expeffect=`expeffect';
 gen basewage=(1+`expreturn')^exper;
 gen wage_policy=basewage*`firebase';
 
  *Compute output to return -- we only need # of teachers from this;
     *Number of teachers under base policy;
       su Surv, meanonly;
       return scalar nteach_base=r(mean);
     *Number of teachers under alternative policy;
       gen weight_policy=entryweight*Surv_policy;
       su weight_policy, meanonly;
       return scalar nteach_policy=r(mean); 
  **A few extra parameters;
   *return scalar firethreshold=`firethreshold';
   *return matrix eta_exit=`eta_exit';
   *return scalar eta_entry=`eta_entry';
*/ 
end;