#delimit;

/******************************************************************************
 * sub_search.do
 * 
 * Runs the search for a starting wage that meets a specified criterion:
 *   numteach:  Gets the same number of total teachers as in baseline.
 *   totsal  :  Gets the same total salary budget as in baseline.
 *   totcost :  Gets the same total budget (salary budget + 
                2*base salary budget*(new num teachers/baseline num teachers))
                as in baseline.
 *                
 * Called by runmodel.do
 *
 * Jesse Rothstein
 * 
 * Modified 4/2/14:  
 *  New optional argument criterionratio(real):  Now search not for baseline
 *  number of teachers/total salary budget/total cost, but for criterionratio
 *  times the baseline.  Default = 1, meaning meet the baseline.
 ******************************************************************************/
 *

cap program drop sub_search;
program define sub_search, rclass;
syntax using, gridpts(integer) simulobs(integer) startwage(real) tol(real) criterion(string) ctype(string)
        elast_entry(string) elast_exit(string) avgelast(real) startwage(real)  [criterionratio(real 1) *];
   local options `"`options' elast_entry(`"`elast_entry'"') elast_exit(`"`elast_exit'"')"';
   di "***";
   di "***Starting iterative search for base pay under alternative contract***";
   di "***Optimizing `criterion'";
   di "***";
   
 *We are going to search for the right base wage in the bonus/firing scheme to 
 *achieve the stated criteria -- either the same number of teachers as in the baseline or the 
 *same total salary;

 if `criterionratio'<=0 {;
 	 di "Error: Criterionratio must be positive.  Specified `criterionratio'";
 	 err;
 };
 if `"`criterion'"'==`"numteach"' local criterion_expression "(r(nteach_policy)/r(nteach_base))-`criterionratio'";
 else if `"`criterion'"'==`"totsal"' local criterion_expression "(r(totsal_policy)/r(totsal_base))-`criterionratio'";
 else if `"`criterion'"'==`"totcost"' 
     local criterion_expression "((r(totsal_policy) + (2*r(totsal_base)*(r(nteach_policy)/r(nteach_base))))/(3*r(totsal_base)))-`criterionratio'";
 else {;
 	 di "Error:  Must specify criterion";
 	 di "Specified |`criterion'|";
 	 err;
 }; 

 if `startwage'<0 {;
 	 di "Cant specify a negative starting wage";
 	 err;
 };
 matrix searchprocess=(0, `startwage', 0, 0, 0, 0, 0, 0, 0);
 matrix colnames searchprocess=i x_r y_r x_s y_s newx newy lambda status;
 
 *Now, do the loop at full size.  Goal is to get nteach_policy = nteach_base;
 *Evaluate the function at the last guess;
  di "Initial function evaluation at starting wage `startwage'";
  local nruns=0;
  local x0=`startwage';
  qui learningmodel_`ctype' `using', gridpts(`gridpts') simulobs(`simulobs') 
                                 `options'
                                 policybase(`x0');
  local nteach_base=r(nteach_base);
  local y0=`criterion_expression';
  local nruns=`nruns'+1;
 *If we are already here, we quit now;
  if abs(`y0') < `tol' {;
   	 di "Initial guess was right!  Basewage = " %12.0g `x0' ". Error = " %8.0g `y0';
   	 local xs=`x0';
   	 local converged=1;
   	 local ys=`y0';
  };
  else {;
  	di "Initial guess:  Basewage = " %12.0g `x0' ". Error = " %8.0g `y0';
    *A good second guess;
     if `"`criterion'"'==`"numteach"' local x1=`x0'*(1 - `y0'/(`elast_entry' + `avgelast'));
     else if `"`criterion'"'==`"totsal"' | `"`criterion'"'==`"totcost"' local x1=`x0'*(1 - `y0'/(1+`elast_entry' + `avgelast'));
    drop _all;
    di "Second function evaluation at second guess `x1'." ;
    qui learningmodel_`ctype' `using', gridpts(`gridpts') simulobs(`simulobs') 
                                   `options'
                                   policybase(`x1');
    local y1=`criterion_expression';
    local nruns=`nruns'+1;
  	di "Second  guess:  Basewage = " %12.0g `x1' ". Error = " %8.0g `y1';
    if abs(`y1')>abs(`y0') {;
  	  *Switch the order;
  	   local oldy0=`y0';
  	   local oldx0=`x0';
  	   local x0=`x1';
  	   local y0=`y1';
  	   local x1=`oldx0';
  	   local y1=`oldy0';
    };
    if abs(`y1') < `tol' {;
     	 di "Second guess was right!  Basewage = " %12.0g `x1' ". Error = " %8.0g `y1';
     	 local xs=`x1';
   	   local converged=1;
   	   local ys=`y1';
    };
    else {;    
      di "Starting loop.";
      *Now, loop.  Goal is to get nteach_policy = nteach_base;
       local converged=0;
       local xs=`x1';
       local ys=`y1';
       local xr=`x0';
       local yr=`y0';
       matrix searchprocess=searchprocess \ (2, `xr', `yr', `xs', `ys', 0, 0, 0, 0);
       matrix colnames searchprocess=i x_r y_r x_s y_s newx newy lambda status;
       *matrix list searchprocess;
       while `nruns'<30 {;
         local deriv=(`ys'-`yr')/(`xs'-`xr');
         local lambda=1;
         matrix rowbase=(`xr', `yr', `xs', `ys');
         foreach lambda in 1 0.5 0.25 0.125 0.0625 0.75 0.875 0.9375 {;
         	 local ++nruns;
           local tryx=`xs' - `lambda'*`ys'/`deriv';
         	 di "Function evaluation `nruns'.  Fullstep " %8.0g `ys'/`deriv' 
         	    ". Lambda = `lambda'  and guess = " %8.0g `tryx';
           drop _all;
           qui learningmodel_`ctype' `using', gridpts(`gridpts') simulobs(`simulobs') 
                                     `options'
                                      policybase(`tryx');
           local tryy=`criterion_expression';
           if abs(`tryy')<abs(`yr') {;
             local lastlambda=`lambda';
             continue, break;
           };
           else {;
           	di "Function evaluation `nruns' finished.  Y = " %8.0g `tryy' ". No improvement.";
           	matrix onerow=(`nruns', `xr', `yr', `xs', `ys', `tryx', `tryy', `lambda', 0);
           	matrix searchprocess=searchprocess \ onerow;
           	*matrix list searchprocess;
           };
         };
         if abs(`tryy')>=abs(`yr') {;
         	 *Failed to find a lambda that improved things;
         	 *Instead, split the difference between x(t-1) and x(t-2);
           foreach lambda in 0.5 0.25 0.75 0.125 0.375 0.625 0.875 {;
           	 local tryx=`xs' + `lambda'*(`xr'- `xs');
           	 local ++nruns;
          	 di "Function evaluation `nruns'.  Stepping backward by " (1-`lambda') " to guess = " %8.0g `tryx';
             drop _all;
             qui learningmodel_`ctype' `using', gridpts(`gridpts') simulobs(`simulobs') 
                                       `options' policybase(`tryx');
             local tryy=`criterion_expression';
             if abs(`tryy')<abs(`yr') {;
               local lastlambda=`lambda';
               continue, break;
             };
             else {;
               matrix onerow=(`nruns', `xr', `yr', `xs', `ys', `tryx', `tryy', -1*(`lambda'), 0);
           	   matrix searchprocess=searchprocess \ onerow;
           	   *matrix list searchprocess;
             };
           };
         };
         if abs(`tryy')<abs(`ys') {;
        	 *We've found a new best;
           di "Function evaluation `nruns' finished.  Y = " %8.0g `tryy' ". New best.";
           matrix onerow=(`nruns', `xr', `yr', `xs', `ys', `tryx', `tryy', `lastlambda', 2);
           matrix searchprocess=searchprocess \ onerow;
           *matrix list searchprocess;
        	 local xr=`xs';
        	 local yr=`ys';
        	 local xs=`tryx';
        	 local ys=`tryy';
         };
         else if abs(`tryy')<abs(`yr') {;
        	 *We've found a new second best;
           di "Function evaluation `nruns' finished.  Y = " %8.0g `tryy' ". New 2nd best.";
           matrix onerow=(`nruns', `xr', `yr', `xs', `ys', `tryx', `tryy', `lastlambda', 1);
           matrix searchprocess=searchprocess \ onerow;
           *matrix list searchprocess;
        	 local xr=`tryx';
        	 local yr=`tryy';
         };
         else {;
        	 *We failed to improve on either of our two best points to date;
        	 *Must be stuck at a nonmonotonicity;
        	 di "Failed to make progress.  Run one last time to recover best to date.";
           qui learningmodel_`ctype' `using', gridpts(`gridpts') simulobs(`simulobs') 
                                     `options' policybase(`xs');
           local tryy=`criterion_expression';
           matrix onerow=(`nruns', `xr', `yr', `xs', `ys', `xs', `tryy', -1, -1);
           matrix searchprocess=searchprocess \ onerow;
        	 *matrix list searchprocess;
        	 continue, break;
         };
         if abs(`tryy') < `tol' {;
        	 di "Converged!  Basewage = " %12.0g `tryx' ". Error = " %8.0g `tryy';
        	 matrix list searchprocess;
        	 local converged=1;
        	 continue, break;
         };
       }; /*End loop over i*/
    }; /*End "else" if didn't find convergence on second try*/
  }; /*End "else" if didn't find convergence on first try*/
  
  return scalar totsal_policy=r(totsal_policy);
  return scalar nteach_policy=r(nteach_policy);
  return scalar totsal_base=r(totsal_base);
  return scalar nteach_base=r(nteach_base);
  return scalar converged=`converged';
  return scalar nruns=`nruns';
  return scalar basewage_policy=`xs';
  return scalar criterion=`ys';
  return add;
end;
